# Functions for the automatic logging of stuff

# Perl code for autogeneration
# sub autogen_C_logging_code
# {
#     # given a hash of arrays of variable names, where the hash
#     # key is the header, autogenerate PRINTF statements
#     my ($self) = @_;
#     my $code = undef;
#     if(defined $self->{_grid_options}->{C_auto_logging} &&
#        ref $self->{_grid_options}->{C_auto_logging} eq 'HASH'
#         )
#     {
#         $code = '';

#         foreach my $header (keys %{$self->{_grid_options}->{C_auto_logging}})
#         {
#             if(ref $self->{_grid_options}->{C_auto_logging}->{$header} eq 'ARRAY')
#             {
#                 $code .= 'PRINTF("'.$header.' ';
#                 foreach my $x (@{$self->{_grid_options}->{C_auto_logging}->{$header}})
#                 {
#                     $code .= '%g ';
#                 }
#                 $code .= '\n"';

#                 foreach my $x (@{$self->{_grid_options}->{C_auto_logging}->{$header}})
#                 {
#                     $code .= ',((double)stardata->'.$x.')';
#                 }
#                 $code .= ');'
#             }
#         }
#     }
#     print "MADE AUTO CODE \n\n************************************************************\n\n$code\n\n************************************************************\n";

#     return $code;
# }

# Which is used in flexi-grid via this:
# $population->set(    C_auto_logging => {
#         'MY_STELLAR_DATA' =>
#             [
#              'model.time',
#              'star[0].mass',
#              'model.probability',
#              'model.dt'
#             ]
#     });



def autogen_C_logging_code(logging_dict):
    """
    Function that autogenerates PRINTF statements for binaryc

    Input:
        dictionary where the key is the header of that logging line and items which are lists of parameters that will be put in that logging line

        example: {'MY_STELLAR_DATA': 
        [
            'model.time',
            'star[0].mass',
            'model.probability',
            'model.dt'
        ']}
    """

    # Check if the input is of the correct form 
    if not type(logging_dict)==dict:
        print("Error: please use a dictionary as input")

    #
    code = ''

    # Loop over dict keys
    for key in logging_dict:
        print('{}'.format(key))

        logging_dict_entry = logging_dict[key]

        # Check if item is of correct type:
        if type(logging_dict_entry)==list:
            print(logging_dict_entry)

            # Construct print statement
            code += 'PRINTF("{}'.format(key)
            code += ' {}'.format('%g '*len(logging_dict_entry))
            code = code.strip()
            code += '\n"'

            # Add format keys
            for param in logging_dict_entry:
                code += ',((double)stardata->{})'.format(param)
            code += ');'

        else:
            print('Error: please use a list for the list of parameters that you want to have logged')


    print(repr(code))

autogen_C_logging_code({'MY_STELLAR_DATA': ['model.time', 'star[0].mass'], 'my_sss2': ['model.time', 'star[1].mass']})


